import sys
from pathlib import Path
import subprocess
import logging
import shutil

from pypdf import PdfWriter

logger = logging.getLogger(__name__)


def _is_mac() -> bool:
    return sys.platform == "darwin"


def _get_libreoffice_path():
    if _is_mac():
        return "/Applications/LibreOffice.app/Contents/MacOS/soffice"
    return "libreoffice"


def has_libreoffice() -> bool:
    if _is_mac():
        return Path(_get_libreoffice_path()).exists()
    return shutil.which("libreoffice") is not None


def doc_to_pdf(path: Path) -> bool:

    tool = _get_libreoffice_path()
    cmd = f"{tool} --headless --convert-to pdf {path} --outdir {path.parent}"
    try:
        subprocess.run(cmd, shell=True, check=True, capture_output=True)
    except subprocess.CalledProcessError as e:
        logger.warn("Error converting pdf:", e.stderr)
        return False
    return True


def merge_pdfs(output: Path, paths: list[Path]):

    merger = PdfWriter()
    for path in paths:
        merger.append(path)
    merger.write(output)



# Generated by Django 5.2.3 on 2025-11-04 17:11

import django.db.models.deletion
import django.utils.timezone
import functools
import ichec_django_core.models.utils
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("ichec_django_core", "0001_initial"),
        (
            "marinerg_facility",
            "0004_equipment_created_at_equipment_updated_at_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="AccessApplication",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("request_start_date", models.DateTimeField(null=True)),
                ("request_end_date", models.DateTimeField(null=True)),
                ("dates_flexible", models.BooleanField(default=False)),
                ("summary", models.FileField(null=True, upload_to="")),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("CREATED", "Created"),
                            ("SUBMITTED", "Submitted"),
                            ("FACILITY", "Awaiting Facility Review"),
                            ("BOARD", "Awaiting Board Review"),
                            ("ACCEPTED", "Accepted"),
                            ("REJECTED", "Rejected"),
                        ],
                        default="CREATED",
                        max_length=10,
                    ),
                ),
                ("submitted", models.DateTimeField(null=True)),
                (
                    "applicant",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "chosen_facility",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="applications",
                        to="marinerg_facility.facility",
                    ),
                ),
                (
                    "facilities",
                    models.ManyToManyField(
                        blank=True,
                        related_name="application_choices",
                        to="marinerg_facility.facility",
                    ),
                ),
                (
                    "form",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.populatedform",
                    ),
                ),
            ],
            options={
                "verbose_name": "Access Application",
                "verbose_name_plural": "Access Application",
            },
        ),
        migrations.CreateModel(
            name="AccessApplicationFacilityReview",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "confirmed_preapplication_discussion",
                    models.BooleanField(default=False),
                ),
                (
                    "confirmed_app_info_in_line_with_discussion",
                    models.BooleanField(default=False),
                ),
                ("supporting_comments", models.TextField(blank=True)),
                ("supporting_documents", models.FileField(null=True, upload_to="")),
                (
                    "application",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accessapplication",
                    ),
                ),
                (
                    "facility",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_facility.facility",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AccessCall",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("title", models.CharField(max_length=200)),
                ("description", models.TextField()),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("OPEN", "Open"),
                            ("CLOSED", "Closed"),
                            ("DRAFT", "Draft"),
                        ],
                        default="DRAFT",
                        max_length=10,
                    ),
                ),
                ("closing_date", models.DateTimeField()),
                (
                    "applications_summary",
                    models.FileField(
                        null=True,
                        upload_to=functools.partial(
                            ichec_django_core.models.utils.content_file_name,
                            *("access_call_applications_summary",),
                            **{},
                        ),
                    ),
                ),
                (
                    "board_chair",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="chaired_access_boards",
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "board_members",
                    models.ManyToManyField(
                        blank=True,
                        related_name="access_boards",
                        to="ichec_django_core.member",
                        verbose_name="list of review board members",
                    ),
                ),
                (
                    "coordinator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "form",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.form",
                    ),
                ),
                (
                    "selectable_facilities",
                    models.ManyToManyField(
                        blank=True,
                        related_name="selectable_access_calls",
                        to="marinerg_facility.facility",
                        verbose_name="Selectable Facilities",
                    ),
                ),
            ],
            options={
                "verbose_name": "Access Call",
                "verbose_name_plural": "Access Calls",
            },
        ),
        migrations.CreateModel(
            name="AccessApplicationBoardReview",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "decision",
                    models.CharField(
                        choices=[("1", "ACCEPT"), ("2", "REJECT")], max_length=10
                    ),
                ),
                ("comments", models.TextField(blank=True)),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
                (
                    "call",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accesscall",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="accessapplication",
            name="call",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="marinerg_test_access.accesscall",
            ),
        ),
        migrations.CreateModel(
            name="AccessCallFacilityReview",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "decision",
                    models.CharField(
                        choices=[("ACCEPT", "Accept"), ("REJECT", "Reject")],
                        max_length=10,
                    ),
                ),
                ("comments", models.TextField(blank=True)),
                (
                    "call",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accesscall",
                    ),
                ),
                (
                    "facility",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_facility.facility",
                    ),
                ),
            ],
            options={
                "verbose_name": "Access Call Facility Review",
                "verbose_name_plural": "Access Call Facility Reviews",
            },
        ),
        migrations.CreateModel(
            name="FacilityTestReport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("report", models.FileField(null=True, upload_to="")),
                ("dataset_uri", models.CharField(blank=True, max_length=200)),
                ("confirmed_complies_data_mgmt", models.BooleanField(default=False)),
                (
                    "application",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accessapplication",
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from datetime import datetime

from ichec_django_core.client import (
    MemberDetail,
    FormCreate,
    FormGroupCreate,
    FormFieldCreate,
    PopulatedFormCreate,
    FormFieldValueCreate,
)

from marinerg_facility.client import FacilityDetail

from .models import (
    AccessCallCreate,
    AccessCallDetail,
    AccessApplicationCreate,
)


def create_access_calls(
    count: int = 10,
    offset: int = 0,
    members: tuple[MemberDetail, ...] = (),
    selectable_facilities: tuple[FacilityDetail, ...] = (),
) -> list[AccessCallCreate]:

    contents = []
    for idx in range(offset, count + offset):
        fields = [
            FormFieldCreate(
                label="Support Needed?",
                key="support_needed",
                field_type="BOOLEAN",
                required=False,
                default="false",
                description="Do you need support?",
            ),
            FormFieldCreate(
                label="Description of Device",
                key="description",
                field_type="TEXT",
                required=True,
                default="",
                description="Give a detailed description of the device",
            ),
        ]

        groups = [FormGroupCreate(fields=fields)]

        contents.append(
            AccessCallCreate(
                title=f"Access Call {idx}",
                description=f"Description of Access Call {idx}",
                status="OPEN",
                public=True,
                use_facility_review=True,
                closing_date=datetime.now(),
                coordinator=members[0].url,
                board_chair=members[1].url,
                board_members=[members[2].url, members[3].url, members[4].url],
                selectable_facilities=[f.url for f in selectable_facilities],
                form=FormCreate(groups=groups),
            )
        )
    return contents


def create_applications(call: AccessCallDetail, count: int = 10, offset: int = 0):

    description_field = call.get_field("description")
    if not description_field:
        raise RuntimeError("Field not found")

    contents = []
    for idx in range(offset, count + offset):
        contents.append(
            AccessApplicationCreate(
                call=call.url,
                form=PopulatedFormCreate(
                    values=[
                        FormFieldValueCreate(
                            field=description_field.id, value="description"
                        )
                    ]
                ),
            )
        )
    return contents



from rest_framework import serializers

from ichec_django_core.serializers.core import SERIALIZER_BASE_FIELDS

from marinerg_test_access.models import (
    FacilityTestReport,
    AccessApplicationFacilityReview,
    AccessApplicationBoardReview,
    AccessApplicationBoardDecision,
)


class FacilityTestReportSerializer(serializers.HyperlinkedModelSerializer):

    report = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="facilitytestreport-report"
    )

    class Meta:
        model = FacilityTestReport
        fields = SERIALIZER_BASE_FIELDS + (
            "creator",
            "report",
            "datasets",
            "confirmed_complies_data_mgmt",
            "application",
            "facility",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS + ("creator", "report")

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.report:
            rep["report"] = None
        return rep


class AccessApplicationFacilityReviewSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = AccessApplicationFacilityReview
        fields = SERIALIZER_BASE_FIELDS + (
            "confirmed_preapplication_discussion",
            "confirmed_app_info_in_line_with_discussion",
            "supporting_comments",
            "supporting_documents",
            "application",
            "facility",
        )


class AccessApplicationBoardReviewSerializer(serializers.HyperlinkedModelSerializer):

    supporting_document = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="accessapplicationboardreview-supporting_document"
    )

    class Meta:
        model = AccessApplicationBoardReview
        fields = SERIALIZER_BASE_FIELDS + (
            "decision",
            "comments",
            "reviewer",
            "application",
            "supporting_document",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS + ("reviewer",)

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.supporting_document:
            rep["supporting_document"] = None
        return rep


class AccessApplicationBoardDecisionSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = AccessApplicationBoardDecision
        fields = SERIALIZER_BASE_FIELDS + (
            "decision",
            "comments",
            "reviewer",
            "application",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS + ("reviewer",)



from django.urls import path

from .views import (
    AccessApplicationViewSet,
    ApplicationFieldDownloadView,
    ApplicationFieldUploadView,
    AccessCallViewSet,
    AccessCallReviewTemplateUploadView,
    AccessCallFieldDownloadView,
    AccessCallFieldUploadView,
    AccessCallFacilityReviewViewSet,
    AccessApplicationFacilityReviewViewSet,
    AccessApplicationBoardReviewViewSet,
    AccessApplicationBoardReviewSupportingDocumentUploadView,
    AccessApplicationBoardDecisionViewSet,
    FacilityTestReportViewSet,
    TestReportUploadView,
)


def register_drf_views(router):
    router.register(r"access_calls", AccessCallViewSet)
    router.register(r"access_call_facility_reviews", AccessCallFacilityReviewViewSet)
    router.register(r"access_applications", AccessApplicationViewSet)
    router.register(
        r"access_application_facility_reviews", AccessApplicationFacilityReviewViewSet
    )
    router.register(
        r"access_application_board_reviews", AccessApplicationBoardReviewViewSet
    )
    router.register(
        r"access_application_board_decisions", AccessApplicationBoardDecisionViewSet
    )
    router.register(r"facility_test_reports", FacilityTestReportViewSet)


urlpatterns = [
    path(
        r"facility_test_reports/<int:pk>/report/upload",
        TestReportUploadView.as_view(),
        name="test_report_upload",
    ),
    path(
        r"access_application_board_reviews/<int:pk>/supporting_document/upload",
        AccessApplicationBoardReviewSupportingDocumentUploadView.as_view(),
        name="supporting_document_upload",
    ),
    path(
        r"access_applications/<int:pk>/fields/<int:value_id>",
        ApplicationFieldDownloadView.as_view(),
        name="application_field_download",
    ),
    path(
        r"access_applications/<int:pk>/fields/<int:value_id>/upload",
        ApplicationFieldUploadView.as_view(),
        name="application_field_upload",
    ),
    path(
        r"access_calls/<int:pk>/fields/<int:value_id>",
        AccessCallFieldDownloadView.as_view(),
        name="access_call_field_download",
    ),
    path(
        r"access_calls/<int:pk>/fields/<int:value_id>/upload",
        AccessCallFieldUploadView.as_view(),
        name="access_call_field_upload",
    ),
    path(
        r"access_calls/<int:pk>/review_template/upload",
        AccessCallReviewTemplateUploadView.as_view(),
        name="access_call_review_template_upload",
    ),
]



# Generated by Django 5.2.3 on 2026-02-16 18:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "marinerg_test_access",
            "0009_accessapplicationboardreview_supporting_document_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="accessapplicationboardreview",
            name="application",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="board_reviews",
                to="marinerg_test_access.accessapplication",
            ),
        ),
    ]



from .access_call import (
    AccessCallFacilityReviewSerializer,
    AccessCallListSerializer,
    AccessCallDetailSerializer,
    AccessCallPublicDetailSerializer,
    AccessCallReviewerDetailSerializer,
)
from .application import (
    AccessApplicationCreateSerializer,
    AccessApplicationDetailSerializer,
)
from .reviews import (
    FacilityTestReportSerializer,
    AccessApplicationFacilityReviewSerializer,
    AccessApplicationBoardReviewSerializer,
    AccessApplicationBoardDecisionSerializer,
)

__all__ = [
    "AccessApplicationSerializer",
    "AccessApplicationMediaSerializer",
    "FacilityTestReportSerializer",
    "AccessApplicationFacilityReviewSerializer",
    "AccessApplicationBoardReviewSerializer",
    "AccessCallListSerializer",
    "AccessCallCreateSerializer",
    "AccessCallDetailSerializer",
    "AccessCallPublicDetailSerializer",
    "AccessCallReviewerDetailSerializer",
    "AccessCallFacilityReviewSerializer",
    "AccessApplicationCreateSerializer",
    "AccessApplicationDetailSerializer",
    "AccessApplicationBoardDecisionSerializer",
]



# Generated by Django 5.2.3 on 2026-02-02 12:12

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_test_access", "0007_alter_facilitytestreport_datasets"),
    ]

    operations = [
        migrations.AddField(
            model_name="accesscall",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="accesscall",
            name="deleted_at",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name="accesscall",
            name="public",
            field=models.BooleanField(default=True),
        ),
    ]



import zipfile
import os
from pathlib import Path
from dataclasses import dataclass, asdict

from django.conf import settings
from django.template.loader import render_to_string

from weasyprint import HTML, CSS

from ..utils.static import find_static
from ..utils.pdf import has_libreoffice, doc_to_pdf, merge_pdfs


@dataclass(frozen=True)
class FormFieldValue:
    type: str
    label: str
    value: str


@dataclass(frozen=True)
class SummaryContext:

    logo_path: str
    first_name: str
    last_name: str
    email: str
    call_title: str
    call_closing_date: str
    call_coordinator_email: str
    submission_date: str
    facilities: list[str]
    dates_flexible: str
    fields: list[FormFieldValue]
    chosen_facility: str | None = None
    start_date: str | None = None
    end_date: str | None = None


def get_summary_context(instance) -> SummaryContext:

    fields = []
    for v in instance.form.values.all():
        match (v.field.field_type):
            case "BOOLEAN":
                value = "Yes" if v.value == "TRUE" else "No"
            case "FILE":
                suffix = Path(v.asset.name).suffix
                value = f"Included file: {v.field.label}{suffix}"
            case _:
                value = v.value

        fields.append(
            FormFieldValue(type=v.field.field_type, label=v.field.label, value=value)
        )

    return SummaryContext(
        logo_path=str(find_static("logo.png")),
        first_name=instance.applicant.first_name,
        last_name=instance.applicant.last_name,
        email=instance.applicant.email,
        call_title=instance.call.title,
        call_coordinator_email=instance.call.coordinator.email,
        call_closing_date=instance.call.closing_date.date(),
        submission_date=instance.submitted.date(),
        facilities=[f.name for f in instance.facilities.all()],
        chosen_facility=(
            instance.chosen_facility.name if instance.chosen_facility else None
        ),
        start_date=(
            instance.request_start_date.date() if instance.request_start_date else None
        ),
        end_date=(
            instance.request_end_date.date() if instance.request_end_date else None
        ),
        dates_flexible="Yes" if instance.dates_flexible else "No",
        fields=fields,
    )


def get_summary_files(instance) -> list[tuple[str, str]]:

    files = []
    for value in instance.form.values.all():
        if value.field.field_type == "FILE" and value.asset:
            suffix = Path(value.asset.name).suffix
            files.append(
                (
                    Path(settings.MEDIA_ROOT) / value.asset.name,
                    f"{value.field.label}{suffix}",
                )
            )
    return files


def make_zip_archive(path: Path, pdf, file_paths: list):

    os.makedirs(path.parent, exist_ok=True)
    with zipfile.ZipFile(path, "w", zipfile.ZIP_DEFLATED, False) as f:
        for path, arcname in file_paths:
            f.write(path, arcname=arcname)
        f.writestr("application_summary.pdf", pdf)


def make_combined_pdf(output_path: Path, pdf, file_paths: list):

    os.makedirs(output_path.parent, exist_ok=True)
    tmp_path = output_path.parent / (str(output_path.stem) + "_temp.pdf")
    with open(tmp_path, "wb") as f:
        f.write(pdf)
        f.flush()

    to_merge = [tmp_path]
    for path, _ in file_paths:
        if path.suffix.lower() in (".doc", ".docx", ".odt"):
            doc_to_pdf(path)
            to_merge.append(Path(str(path).split(".")[0] + ".pdf"))
        elif path.suffix.lower() == ".pdf":
            to_merge.append(path)
    merge_pdfs(output_path, to_merge)
    # tmp_path.unlink()


def generate_summary(instance, output_field: str = "summary"):

    html = render_to_string(
        "marinerg_test_access/application_summary.html",
        asdict(get_summary_context(instance)),
    )

    pdf = HTML(string=html).write_pdf(stylesheets=[CSS(find_static("style.css"))])

    field_path = f"{instance._meta.model_name}_{output_field}"
    if has_libreoffice():
        filename = f"{field_path}_{instance.id}.pdf"
        make_combined_pdf(
            Path(settings.MEDIA_ROOT) / field_path / filename,
            pdf,
            get_summary_files(instance),
        )
    else:
        filename = f"{field_path}_{instance.id}.zip"
        make_zip_archive(
            Path(settings.MEDIA_ROOT) / field_path / filename,
            pdf,
            get_summary_files(instance),
        )

    instance.summary.name = os.path.join(field_path, filename)



from pathlib import Path
import os
import argparse
import logging

import yaml

from ichec_django_core.client import PortalClient, Paginated, MemberDetail, FormDetail

from marinerg_facility.client import FacilityDetail
from marinerg_data_access.client import DatasetDetail

from .models import AccessCallCreate

logger = logging.getLogger(__name__)


class MarinergTestAccessClient(PortalClient):

    def __init__(
        self,
        username: str | None = None,
        password: str | None = None,
        endpoint: str = "http://localhost:8000/",
    ):
        super().__init__(endpoint, username, password)

    def get_facilities(self, search_term="", page: int = 1) -> Paginated:
        return self.get_items(FacilityDetail, page)

    def get_members(self) -> Paginated:
        return self.get_items(MemberDetail)

    def get_datasets(self, query: dict | None = None, page: int = 1) -> Paginated:
        return self.get_items(DatasetDetail, page)

    def get_form_field_id(self, field_key: str, form: FormDetail):
        for group in form.groups:
            for field in group.fields:
                if field.key == field_key:
                    return field.id
        raise RuntimeError("Requested field not found")

    def create_access_call(
        self, call: AccessCallCreate, field_files: tuple[tuple[str, ...], ...] = ()
    ):
        response = self.create_item(call)
        for field_key, file_path in field_files:

            with open(file_path, "rb") as f:
                content = f.read()

            field_id = self.get_form_field_id(field_key, response.form)
            self.put_file(
                f"form_fields/{field_id}/template/upload", content, str(file_path)
            )
        return response


def commom_init(args):

    username = os.getenv("MARINERG_USERNAME", "consortium_admin")
    password = os.getenv("MARINERG_PASSWORD", "abc123")
    endpoint = os.getenv("MARINERG_ENDPOINT", "http://localhost:8000/")
    return username, password, endpoint


def cli_access_call_create(args):
    username, password, endpoint = commom_init(args)

    client = MarinergTestAccessClient(username, password, endpoint)

    with open(args.path.resolve(), "r", encoding="utf-8") as f:
        call_yaml = yaml.safe_load(f)

    members = client.get_members()
    call_yaml["status"] = "DRAFT"
    call_yaml["coordinator"] = members.results[0].url
    call_yaml["board_chair"] = members.results[1].url

    if args.media_dir:
        media_dir = Path(args.media_dir).resolve()
    else:
        media_dir = Path(os.getcwd())

    media_files = []
    for group in call_yaml["form"]["groups"]:
        if "fields" not in group:
            continue
        for field in group["fields"]:
            if field["field_type"] == "FILE" and field["template"]:
                media_files.append((field["key"], media_dir / field["template"]))
            field["template"] = None

    call = AccessCallCreate(**call_yaml)

    client.create_access_call(call, tuple(media_files))


def main_cli():
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(required=True)

    access_call_parser = subparsers.add_parser("access_call")
    access_call_subparsers = access_call_parser.add_subparsers(required=True)

    access_call_create_parser = access_call_subparsers.add_parser("create")

    access_call_create_parser.add_argument(
        "path",
        type=Path,
        help="Path to the call file",
    )

    access_call_create_parser.add_argument(
        "--media_dir",
        type=str,
        default="",
        help="Path to a directory with media to include",
    )

    access_call_parser.set_defaults(func=cli_access_call_create)

    args = parser.parse_args()
    args.func(args)



from pathlib import Path
from django.test import TestCase
from django.core import mail

from ichec_django_core.models import Member

from ichec_django_core.test import (
    setup_default_users_and_groups,
)

from marinerg_test_access.test import (
    setup_default_access_calls,
    setup_application,
)


class ApplicationSummaryTestCase(TestCase):

    def setUp(self):
        setup_default_users_and_groups()
        public, _, _ = setup_default_access_calls()

        test_data_dir = Path(__file__).parent / "data"
        self.application = setup_application(
            public,
            "test_application",
            [
                ("application_form", test_data_dir / "test_application.pdf"),
                ("application_form_docx", test_data_dir / "Sample_docx.docx"),
                ("application_form_doc", test_data_dir / "Sample_doc.doc"),
            ],
        )

    def test_summary_generation(self):
        self.application.status = "SUBMITTED"
        self.application.save()



# Generated by Django 5.2.3 on 2025-11-19 15:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "marinerg_test_access",
            "0002_accessapplicationboardreview_application_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="accessapplicationboarddecision",
            old_name="creator",
            new_name="reviewer",
        ),
        migrations.RenameField(
            model_name="accessapplicationboardreview",
            old_name="creator",
            new_name="reviewer",
        ),
        migrations.RemoveField(
            model_name="accessapplicationboardreview",
            name="call",
        ),
    ]



# Generated by Django 5.2.3 on 2025-11-19 17:06

import django.db.models.deletion
import functools
import ichec_django_core.models.utils
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_facility", "0007_equipmenttag_facilitytag_equipment_tags_and_more"),
        ("marinerg_test_access", "0004_remove_accessapplicationboarddecision_call"),
    ]

    operations = [
        migrations.AddField(
            model_name="facilitytestreport",
            name="facility",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="marinerg_facility.facility",
            ),
        ),
        migrations.AlterField(
            model_name="facilitytestreport",
            name="dataset_uri",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="facilitytestreport",
            name="report",
            field=models.FileField(
                null=True,
                upload_to=functools.partial(
                    ichec_django_core.models.utils.content_file_name, *("report",), **{}
                ),
            ),
        ),
    ]



from rest_framework import permissions, viewsets
from rest_framework.decorators import action

from ichec_django_core.models import Member
from ichec_django_core.views import ObjectFileUploadView, BaseModelViewSet

from ..models import (
    FacilityTestReport,
    AccessApplicationFacilityReview,
    AccessApplicationBoardReview,
    AccessApplicationBoardDecision,
)

from ..serializers import (
    FacilityTestReportSerializer,
    AccessApplicationFacilityReviewSerializer,
    AccessApplicationBoardReviewSerializer,
    AccessApplicationBoardDecisionSerializer,
)


class FacilityTestReportViewSet(BaseModelViewSet):
    queryset = FacilityTestReport.objects.all().order_by("id")
    serializer_class = FacilityTestReportSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]

    @action(detail=True, url_path="report", url_name="report")
    def report(self, request, pk=None):
        return self.download(request, "report")

    def perform_create(self, serializer):
        serializer.save(creator=Member.objects.get(id=self.request.user.id))


class TestReportUploadView(ObjectFileUploadView):
    model = FacilityTestReport
    queryset = FacilityTestReport.objects.all()
    file_field = "report"
    extensions = (".odt", ".doc", ".docx", ".pdf")
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]


class AccessApplicationFacilityReviewViewSet(viewsets.ModelViewSet):
    queryset = AccessApplicationFacilityReview.objects.all().order_by("id")
    serializer_class = AccessApplicationFacilityReviewSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]


class AccessApplicationBoardReviewViewSet(viewsets.ModelViewSet):
    queryset = AccessApplicationBoardReview.objects.all().order_by("id")
    serializer_class = AccessApplicationBoardReviewSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]

    @action(detail=True, url_path="supporting_document", url_name="supporting_document")
    def supporting_document(self, request, pk=None):
        return self.download(request, "supporting_document")

    def perform_create(self, serializer):
        serializer.save(reviewer=Member.objects.get(id=self.request.user.id))


class AccessApplicationBoardReviewSupportingDocumentUploadView(ObjectFileUploadView):
    model = AccessApplicationBoardReview
    queryset = AccessApplicationBoardReview.objects.all()
    file_field = "supporting_document"
    extensions = (".odt", ".doc", ".docx", ".pdf")
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]


class AccessApplicationBoardDecisionViewSet(viewsets.ModelViewSet):
    queryset = AccessApplicationBoardDecision.objects.all().order_by("id")
    serializer_class = AccessApplicationBoardDecisionSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]

    def perform_create(self, serializer):
        serializer.save(reviewer=Member.objects.get(id=self.request.user.id))



from django.utils import timezone

from ichec_django_core.models import Member
from ichec_django_core.client import MemberDetail, FormFieldCreate

from ichec_django_core.test import (
    AuthAPITestCase,
    add_group_permissions,
    setup_default_users_and_groups,
)

from marinerg_test_access.client.resources import create_access_calls
from marinerg_test_access.models import AccessCall
from marinerg_test_access.test import setup_default_access_calls


class TestAccessCallView(AuthAPITestCase):

    def _get_review_board(self) -> list[MemberDetail]:
        return [
            self.get_member_detail(n).body
            for n in ["coordinator", "chair", "reviewer1", "reviewer2", "reviewer3"]
        ]

    def setUp(self):
        self.url = "/api/access_calls/"

        setup_default_users_and_groups()
        add_group_permissions(
            "admins",
            AccessCall,
            [
                "view_accesscall",
                "change_accesscall",
                "add_accesscall",
                "delete_accesscall",
            ],
        )
        add_group_permissions("members", AccessCall, ["view_accesscall"])

        self.internal, self.public, self.inactive = setup_default_access_calls()
        self.template = create_access_calls(1, members=self._get_review_board())[0]

    def get_template(self, username: str = "admin_user") -> dict:
        return self.template.model_copy(
            update={
                "coordinator": self.get_member_url(username),
                "board_chair": self.get_member_url(username),
            }
        )

    def test_list_access(self):

        scenarios = (
            ["", 1, 200, "Public can list public items"],
            ["regular_user", 1, 200, "Registered can list public items"],
            ["member", 2, 200, "Member can list all items"],
            ["coordinator", 2, 200, "Coordinator can list item"],
            ["reviewer1", 2, 200, "Reviewer can list item"],
            ["chair", 2, 200, "Chair can list item"],
        )
        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            self.assertEqual(response.body.count, count, msg)

    def xtest_detail_access(self):

        public = self.public.id
        internal = self.internal.id
        scenarios = (
            ["", public, "public", 200, "Public can view public item"],
            ["", internal, "public", 404, "Public can't view internal item"],
            ["regular_user", public, "public", 200, "Registered can view public item"],
            [
                "regular_user",
                internal,
                "public",
                404,
                "Registered can't view internal item",
            ],
            ["member", public, "public", 200, "Member can view public item"],
            ["member", internal, "public", 200, "Member can view internal item"],
            ["coordinator", internal, "", 200, "Coordinator can view internal item"],
            ["chair", internal, "", 200, "Chair can view internal item"],
            ["reviewer1", internal, "reviewer", 200, "Reviewer can view internal item"],
        )

        for user, item, role, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item, role), status, msg)
            else:
                self.assert_status(self.detail(item, role), status, msg)

    def test_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 403, "Registered can't create, no perm"],
            ["member", 403, "Member can't create, no perm"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(
                    self.auth_create(user, self.get_template()), status, msg
                )
            else:
                self.assert_status(self.create(self.get_template()), status, msg)

    def test_update_permissions(self):
        created = self.assert_201(
            self.auth_create("admin_user", self.get_template())
        ).body

        new_field = FormFieldCreate(
            label="Requires More Support",
            key="requires_more_support",
            description="Do you require more facility support?",
            field_type="BOOLEAN",
            default="false",
        )
        updated_group = created.form.groups[0].model_copy(
            update={"fields": created.form.groups[0].fields + [new_field]}
        )

        updated = created.model_copy(
            update={
                "description": "Modified description of my access call",
                "form": created.form.model_copy(update={"groups": [updated_group]}),
            }
        )

        updated = self.assert_200(self.auth_update("admin_user", created.id, updated))

        self.assertTrue(
            updated.body.description == "Modified description of my access call"
        )
        self.assertTrue(len(updated.body.form.groups[0].fields) == 3)



from pathlib import Path
import os

from ichec_django_core.settings import *
from ichec_django_core import settings

SECRET_KEY = "fake-key"

MEDIA_ROOT = Path(__file__).parent / "test_output/mediafiles/"
os.makedirs(MEDIA_ROOT, exist_ok=True)

INSTALLED_APPS = [
    "marinerg_facility.apps.MarinergFacilityConfig",
    "marinerg_test_access.apps.MarinergTestAccessConfig",
    "marinerg_data_access.apps.MarinergDataAccessConfig",
]
INSTALLED_APPS.extend(settings.INSTALLED_APPS)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "db.sqlite3",
        "USER": "user",
        "PASSWORD": "password",
    }
}

ROOT_URLCONF = "app.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(MEDIA_ROOT, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]



import uuid
import os
from pathlib import Path

from django.http import Http404
from django.core.files.storage import FileSystemStorage
from django.conf import settings
from django.db.models import Q

from rest_framework.viewsets import ModelViewSet
from rest_framework import permissions
from rest_framework.decorators import action
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from rest_framework.parsers import FileUploadParser

from ichec_django_core.models import Member, FormFieldValue
from ichec_django_core.views import (
    ObjectFileDownloadView,
    SearchableModelViewSet,
    CustomDjangoModelPermissions,
)

from marinerg_test_access.models import AccessApplication
from marinerg_test_access.serializers import (
    AccessApplicationCreateSerializer,
    AccessApplicationDetailSerializer,
)


def is_on_access_board(application, user):
    if user.id == application.call.board_chair.id:
        return True

    for member in application.call.board_members.all():
        if member.id == user.id:
            return True
    return False


def is_call_coordinator(application, user):
    return application.call.coordinator.id == user.id


def is_on_board(application, user):
    return is_call_coordinator(application, user) or is_on_access_board(
        application, user
    )


class ApplicationPermissions(CustomDjangoModelPermissions):

    intercept_methods: list = ["GET", "PUT", "POST"]
    safe_methods: list = ["HEAD", "OPTIONS"]

    def object_perm_check(self, obj, request) -> bool:

        # All authenticated users can post
        if request.method == "POST":
            return True

        # Applicant can get, put, post
        if obj.applicant.id == request.user.id:
            return True

        # Review board can get
        if is_on_board(obj, request.user) and request.method == "GET":
            return True

        # Use Django permissions for everything else
        return False


class AccessApplicationViewSet(SearchableModelViewSet):
    model = AccessApplication
    queryset = AccessApplication.objects.all()
    serializer_class = AccessApplicationDetailSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        ApplicationPermissions,
    ]

    ordering: tuple[str, ...] = ("created_at",)
    search_fields = ["applicant__usename"]

    serializers = {
        "retrieve": AccessApplicationDetailSerializer,
        "list": AccessApplicationDetailSerializer,
        "create": AccessApplicationCreateSerializer,
        "update": AccessApplicationDetailSerializer,
        "partial_update": AccessApplicationDetailSerializer,
    }

    filterset_fields = (
        "applicant__id",
        "call__id",
        "facilities__id",
        "chosen_facility__id",
        "status",
    )

    @action(detail=True, url_path="summary", url_name="summary")
    def summary(self, request, pk=None):
        return self.download(request, "summary")

    def perform_create(self, serializer):
        serializer.save(applicant=Member.objects.get(id=self.request.user.id))

    def get_queryset(self):
        """
        If user has Application view perm return all applications
        Otherwise only return if they are the applicant or
        member of the review board
        """

        if self.has_model_view_permission():
            return ModelViewSet.get_queryset(self)

        queryset = ModelViewSet.get_queryset(self)
        queryset = queryset.filter(
            Q(applicant__id=self.request.user.id)
            | Q(call__board_chair__id=self.request.user.id)
            | Q(call__board_members__id=self.request.user.id)
            | Q(call__coordinator__id=self.request.user.id)
        ).distinct()
        return queryset


class ApplicationFieldDownloadView(ObjectFileDownloadView):

    model = AccessApplication
    permission_classes = [
        permissions.IsAuthenticated,
        ApplicationPermissions,
    ]

    def get_file(self):
        if not self.value.asset:
            raise Http404
        return self.value.asset

    def get_basename(self):
        ext = self.value.asset.name.split(".")[-1]
        return f"application_{self.value.field.key}.{ext}"

    def get(self, request, *args, **kwargs):
        self.authenticate(request)
        self.has_permission(request)

        self.object = self.get_object()
        self.value = FormFieldValue.objects.get(id=kwargs["value_id"])
        return super(ObjectFileDownloadView, self).get(request, *args, **kwargs)


class ApplicationFieldUploadView(GenericAPIView):

    model = AccessApplication
    queryset = AccessApplication.objects.all()
    parser_classes = [FileUploadParser]
    permission_classes = [
        permissions.IsAuthenticated,
        ApplicationPermissions,
    ]
    extensions = (".odt", ".doc", ".docx", ".pdf")

    def put(self, request, *args, **kwargs):
        value = FormFieldValue.objects.get(id=kwargs["value_id"])

        media_root = Path(settings.MEDIA_ROOT)
        instance = self.get_object()
        storage_dir = Path(f"{self.model._meta.model_name}/{instance.id}/field_values")
        os.makedirs(media_root / storage_dir, exist_ok=True)

        filename = request.data["file"].name

        _, ext = os.path.splitext(filename)
        if ext not in self.extensions:
            return Response(
                {"reason": "File extension not supported"},
                status=400,
            )

        file_path = f"{uuid.uuid4()}{ext}"
        written_path = FileSystemStorage(location=media_root / storage_dir).save(
            file_path, request.data["file"]
        )

        value.asset.original_file_name = filename
        value.asset.name = str(storage_dir / written_path)
        value.save()

        return Response(status=204)



import functools

from django.db import models

from ichec_django_core.models import Member, TimesStampMixin, content_file_name

from marinerg_data_access.models import Dataset
from marinerg_facility.models import Facility

from .application import AccessApplication


class FacilityTestReport(TimesStampMixin):

    creator = models.ForeignKey(Member, on_delete=models.CASCADE)
    report = models.FileField(
        null=True, upload_to=functools.partial(content_file_name, "report")
    )

    datasets = models.ManyToManyField(
        Dataset,
        verbose_name="Datasets",
        related_name="test_reports",
        blank=True,
    )

    confirmed_complies_data_mgmt = models.BooleanField(default=False)
    application = models.ForeignKey(AccessApplication, on_delete=models.CASCADE)
    facility = models.ForeignKey(Facility, on_delete=models.CASCADE, null=True)


class AccessApplicationFacilityReview(TimesStampMixin):

    confirmed_preapplication_discussion = models.BooleanField(default=False)
    confirmed_app_info_in_line_with_discussion = models.BooleanField(default=False)
    supporting_comments = models.TextField(blank=True)
    supporting_documents = models.FileField(null=True)
    facility = models.ForeignKey(Facility, on_delete=models.CASCADE)
    application = models.ForeignKey(AccessApplication, on_delete=models.CASCADE)


class AccessApplicationBoardReview(TimesStampMixin):
    class Decision(models.TextChoices):
        ACCEPT = "1", "ACCEPT"
        REJECT = "2", "REJECT"

    decision = models.CharField(max_length=10, choices=Decision.choices)
    comments = models.TextField(blank=True)
    application = models.ForeignKey(
        AccessApplication,
        on_delete=models.CASCADE,
        null=True,
        related_name="board_reviews",
    )
    reviewer = models.ForeignKey(Member, on_delete=models.CASCADE)
    supporting_document = models.FileField(null=True)

    def save(self, force_insert=False, force_update=False, *args, **kwargs):

        super().save(force_insert, force_update, *args, **kwargs)

        if len(self.application.call.board_members.all()) == len(
            self.application.board_reviews.all()
        ):
            self.application.status = "DECISION"
            self.application.save()


class AccessApplicationBoardDecision(TimesStampMixin):
    class Decision(models.TextChoices):
        ACCEPT = "1", "ACCEPT"
        REJECT = "2", "REJECT"

    decision = models.CharField(max_length=10, choices=Decision.choices)
    comments = models.TextField(blank=True)
    application = models.ForeignKey(
        AccessApplication, on_delete=models.CASCADE, null=True
    )
    reviewer = models.ForeignKey(Member, on_delete=models.CASCADE)

    def save(self, force_insert=False, force_update=False, *args, **kwargs):

        if self.decision == "1":
            self.application.status = "ACCEPTED"
        else:
            self.application.status = "REJECTED"
        self.application.save()

        super().save(force_insert, force_update, *args, **kwargs)



from ichec_django_core.test import (
    AuthAPITestCase,
    add_group_permissions,
    setup_default_users_and_groups,
)

from marinerg_test_access.models import AccessApplication
from marinerg_test_access.client import AccessCallPublicDetail
from marinerg_test_access.client.resources import create_applications
from marinerg_test_access.test import (
    setup_default_access_calls,
    setup_application,
)


class TestApplicationView(AuthAPITestCase):

    def setUp(self):
        self.url = "/api/access_applications/"
        setup_default_users_and_groups()

        add_group_permissions(
            "regular_users",
            AccessApplication,
            [
                "add_accessapplication",
            ],
        )

        add_group_permissions(
            "admins",
            AccessApplication,
            [
                "view_accessapplication",
                "add_accessapplication",
            ],
        )

        self.internal, self.public, _ = setup_default_access_calls()
        self.application1 = setup_application(self.public, "applicant")
        self.application2 = setup_application(self.public, "applicant2")

        call = self.raw_get(f"/api/access_calls/{self.public.id}/")
        call = AccessCallPublicDetail(**call.body)
        self.template = create_applications(call, 1)[0]

    def xtest_list_access(self):

        scenarios = (
            ["", 0, 401, "Public can't see any applications"],
            ["regular_user", 0, 200, "Registered user sees no applications"],
            ["member", 0, 200, "Member sees no applications"],
            ["coordinator", 2, 200, "Coordinator can list application"],
            ["reviewer1", 2, 200, "Reviewer can list application"],
            ["chair", 2, 200, "Chair can list application"],
            ["applicant", 1, 200, "Applicant can list application"],
            ["admin_user", 2, 200, "Admin can list application"],
        )
        for user, count, status, msg in scenarios:
            if user:
                response = self.assert_status(self.auth_list(user), status, msg)
            else:
                response = self.assert_status(self.public_list(), status, msg)

            if status < 400:
                self.assertEqual(response.body.count, count, msg)

    def xtest_detail_access(self):

        app = self.application1.id
        scenarios = (
            ["", app, 401, "Public can't view item"],
            ["regular_user", app, 404, "Registered can't view item"],
            ["member", app, 404, "Member can't view item"],
            ["applicant", app, 200, "Applicant can view item"],
            ["coordinator", app, 200, "Coordinator can view item"],
            ["chair", app, 200, "Chair can view item"],
            ["reviewer1", app, 200, "Reviewer can view item"],
            ["admin_user", app, 200, "Admin can view item"],
        )

        for user, item, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item), status, msg)
            else:
                self.assert_status(self.detail(item), status, msg)

    def xtest_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 201, "Registered can create"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_create(user, self.template), status, msg)
            else:
                self.assert_status(self.create(self.template), status, msg)

    def xtest_update_regular_user(self):
        response = self.auth_create("regular_user", self.template)
        self.assert_201(response)

        created = response.body
        updated = created.model_copy(update={"dates_flexible": False})

        updated = self.auth_update("regular_user", created.id, updated)
        # self.assert_200(updated)

    def xtest_submit(self):
        response = self.auth_create("regular_user", self.template)
        self.assert_201(response)
        created = response.body

        updated = created.model_copy(update={"status": "SUBMITTED"})
        updated = self.auth_update("regular_user", created.id, updated)
        # self.assert_200(updated)



from rest_framework import serializers
from rest_framework.reverse import reverse

from ichec_django_core.models import Member
from ichec_django_core.serializers import (
    NestedHyperlinkedModelSerializer,
    PopulatedFormSerializer,
)

from marinerg_test_access.models import AccessApplication


class AccessApplicationBaseSerializer(NestedHyperlinkedModelSerializer):

    form = PopulatedFormSerializer()

    class Meta:
        model = AccessApplication
        fields = (
            "facilities",
            "request_start_date",
            "request_end_date",
            "dates_flexible",
            "call",
            "form",
            "status",
        )


class AccessApplicationDetailSerializer(AccessApplicationBaseSerializer):

    call_title = serializers.CharField(
        source="call.title", required=False, read_only=True
    )
    applicant_username = serializers.CharField(
        source="applicant.username", required=False, read_only=True
    )

    summary = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="accessapplication-summary"
    )

    is_selectable_facility_member = serializers.SerializerMethodField()
    is_selected_facility_member = serializers.SerializerMethodField()

    def get_is_selectable_facility_member(self, obj):
        user = self.context.get("request").user

        members = Member.objects.filter(id=user.id)
        if not members:
            return False

        org_ids = [org.id for org in members.first().organizations.all()]
        selectable_ids = [f.id for f in obj.facilities.all()]
        return any(org_id in selectable_ids for org_id in org_ids)

    def get_is_selected_facility_member(self, obj):
        user = self.context.get("request").user

        if not obj.chosen_facility:
            return False

        members_ids = [m.id for m in obj.chosen_facility.members.all()]
        return user.id in members_ids

    def update(self, instance, validated_data):
        form = validated_data.pop("form")

        instance = super().update(instance, validated_data)

        serializer = PopulatedFormSerializer()
        serializer.update(instance.form, form)

        return instance

    def to_representation(self, instance):

        rep = super().to_representation(instance)
        if not instance.summary:
            rep["summary"] = None

        for value in rep["form"]["values"]:
            if value["asset"]:
                value["asset"] = reverse(
                    "application_field_download",
                    args=[instance.id, value["id"]],
                    request=self.context["request"],
                )
            else:
                value["asset"] = None
        return rep

    class Meta:
        model = AccessApplication
        fields = (
            tuple(AccessApplicationBaseSerializer.Meta.fields)
            + NestedHyperlinkedModelSerializer.base_fields
            + (
                "chosen_facility",
                "summary",
                "applicant",
                "call",
                "submitted",
                "call_title",
                "applicant_username",
                "is_selectable_facility_member",
                "is_selected_facility_member",
            )
        )
        read_only_fields = NestedHyperlinkedModelSerializer.base_fields + (
            "submitted",
            "applicant",
            "call_title",
            "summary",
            "applicant_username",
            "is_selectable_facility_member",
            "is_selected_facility_member",
        )


class AccessApplicationCreateSerializer(AccessApplicationBaseSerializer):

    class Meta:
        model = AccessApplication
        fields = AccessApplicationBaseSerializer.Meta.fields

    def create(self, validated_data):
        request = self.context.get("request", None)

        if (
            AccessApplication.objects.filter(
                call__id=validated_data["call"].id, applicant__id=request.user.id
            ).count()
            > 0
        ):
            raise serializers.ValidationError(
                {"call": "Application exists for this call"}
            )

        form = validated_data.pop("form")
        form_instance = PopulatedFormSerializer().create(form)

        many_to_many = self.pop_many_to_many(validated_data)
        instance = AccessApplication.objects.create(
            form=form_instance, **validated_data
        )
        self.add_many_to_many(instance, many_to_many)
        return instance

    def to_representation(self, instance):
        return AccessApplicationDetailSerializer(
            context=self.context
        ).to_representation(instance)



from pathlib import Path

from django.utils import timezone

from ichec_django_core.models import (
    Member,
    Form,
    FormGroup,
    FormField,
    PopulatedForm,
    FormFieldValue,
)

from marinerg_test_access.models import AccessCall, AccessApplication


def _setup_access_call_members():

    coordinator = Member.objects.create(
        username="coordinator",
        first_name="Access Call",
        last_name="Coordinator",
        email="coordinator@example.com",
    )
    chair = Member.objects.create(
        username="chair",
        first_name="Access Call",
        last_name="Board Member",
        email="chair@example.com",
    )
    reviewer1 = Member.objects.create(
        username="reviewer1",
        first_name="Reviewer 1",
        last_name="Board Member",
        email="chair@example.com",
    )

    reviewer2 = Member.objects.create(
        username="reviewer2",
        first_name="Reviewer2",
        last_name="Board Member",
        email="chair@example.com",
    )

    reviewer3 = Member.objects.create(
        username="reviewer3",
        first_name="Reviewer3",
        last_name="Board Member",
        email="chair@example.com",
    )
    return coordinator, chair, reviewer1, reviewer2, reviewer3


def _setup_form():
    form = Form.objects.create()

    group = FormGroup.objects.create(form=form)

    FormField.objects.create(
        group=group,
        label="Sample Bool Field?",
        key="bool_field",
        required=True,
        field_type="BOOLEAN",
    )

    FormField.objects.create(
        group=group,
        label="Description of device",
        key="description",
        required=True,
        field_type="TEXT",
    )

    FormField.objects.create(
        group=group,
        label="Application Form",
        key="application_form",
        required=True,
        field_type="FILE",
    )

    FormField.objects.create(
        group=group,
        label="Application Form Docx",
        key="application_form_docx",
        required=True,
        field_type="FILE",
    )

    FormField.objects.create(
        group=group,
        label="Application Form Doc",
        key="application_form_doc",
        required=True,
        field_type="FILE",
    )
    return form


def _setup_access_call(public: bool, active: bool, members):

    coordinator, chair, reviewer1, reviewer2, reviewer3 = members

    call = AccessCall.objects.create(
        title="Access call",
        description="Description of access call",
        status="OPEN",
        closing_date=timezone.now(),
        coordinator=coordinator,
        board_chair=chair,
        form=_setup_form(),
        public=public,
        active=active,
    )
    call.board_members.set([reviewer1, reviewer2, reviewer3])
    return call


def setup_default_access_calls():

    members = _setup_access_call_members()
    internal = _setup_access_call(False, True, members)
    public = _setup_access_call(True, True, members)
    inactive = _setup_access_call(True, False, members)
    return internal, public, inactive


def _setup_appliction_form(call, assets: list[tuple[str, Path]] | None = None):

    form = PopulatedForm.objects.create()

    FormFieldValue.objects.create(
        value="TRUE", field=call.get_field_by_key("bool_field"), form=form
    )

    if assets:
        for key, path in assets:

            file_field = FormFieldValue.objects.create(
                field=call.get_field_by_key(key), form=form
            )
            file_field.asset.original_filename = path.name
            file_field.asset.name = str(path)
            file_field.save()
    return form


def setup_application(
    call, applicant="test_applicant", assets: list[tuple[str, Path]] | None = None
):

    applicant = Member.objects.create(
        username=applicant,
        first_name="Test",
        last_name="Applicant",
        email=applicant + "@example.com",
    )
    return AccessApplication.objects.create(
        applicant=applicant, call=call, form=_setup_appliction_form(call, assets)
    )



import uuid
import os
from pathlib import Path

from django.conf import settings
from django.http import Http404
from django.core.files.storage import FileSystemStorage
from django.db.models import Q

from rest_framework.viewsets import ModelViewSet
from rest_framework import permissions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.generics import GenericAPIView
from rest_framework.parsers import FileUploadParser

from ichec_django_core.models import FormField
from ichec_django_core.views import (
    SearchableModelViewSet,
    ObjectFileDownloadView,
    ObjectFileUploadView,
    PublicViewOwnerFullOrDjangoModelPermissions,
    OwnerFullOrDjangoModelPermissions,
)

from marinerg_test_access.models import AccessCall, AccessCallFacilityReview
from marinerg_test_access.serializers import (
    AccessCallListSerializer,
    AccessCallDetailSerializer,
    AccessCallPublicDetailSerializer,
    AccessCallReviewerDetailSerializer,
    AccessCallFacilityReviewSerializer,
)


class CoordinatorEditOrDjangoModelPermissions(OwnerFullOrDjangoModelPermissions):
    owner_field = "coordinator"


class PublicViewCoordinatorEditOrDjangoModelPermissions(
    PublicViewOwnerFullOrDjangoModelPermissions
):
    owner_field = "coordinator"


def is_reviewer(call, request):
    for member in call.board_members.all():
        if member.id == request.user.id:
            return True
    return False


def is_coordinator_or_chair(call, request):
    return (
        call.coordinator.id == request.user.id or call.board_chair.id == request.user.id
    )


class AccessCallViewSet(SearchableModelViewSet):
    model = AccessCall
    queryset = AccessCall.objects.filter(active=True)
    serializer_class = AccessCallListSerializer

    serializers = {
        "retrieve": AccessCallDetailSerializer,
        "public_retrieve": AccessCallPublicDetailSerializer,
        "reviewer_retrieve": AccessCallReviewerDetailSerializer,
        "list": AccessCallListSerializer,
        "create": AccessCallDetailSerializer,
        "update": AccessCallDetailSerializer,
        "partial_update": AccessCallDetailSerializer,
    }

    permission_classes = [
        PublicViewCoordinatorEditOrDjangoModelPermissions,
    ]

    ordering_fields = ("status",)
    ordering: tuple[str, ...] = ("-closing_date",)
    search_fields = ["title"]

    @action(
        detail=True,
        url_path="applications_summary",
        url_name="applications_summary",
        permission_classes=[CoordinatorEditOrDjangoModelPermissions],
    )
    def applications_summary(self, request, pk=None):
        return self.download(request, "applications_summary")

    @action(
        detail=True,
        url_path="review_template",
        url_name="review_template",
        permission_classes=[CoordinatorEditOrDjangoModelPermissions],
    )
    def review_template(self, request, pk=None):
        return self.download(request, "review_template")

    def get_queryset(self):
        """
        Public users can only see public calls
        Registered users can see public calls or ones they are on the board of
        Members can see all calls
        Users with view permission can see all calls
        """

        queryset = ModelViewSet.get_queryset(self)

        if not self.request.user.is_authenticated:
            queryset = queryset.filter(public=True)
        elif not self.has_model_view_permission():
            queryset = queryset.filter(
                Q(public=True)
                | Q(board_members__id=self.request.user.id)
                | Q(board_chair__id=self.request.user.id)
                | Q(coordinator__id=self.request.user.id)
            ).distinct()
        return queryset

    def get_serializer_class(self):

        if self.action == "retrieve":
            prefix = ""
            if not self.request.user.is_authenticated:
                prefix = "public_"
            elif self.has_model_view_permission():
                prefix = ""
            elif is_coordinator_or_chair(self.get_object(), self.request):
                prefix = ""
            elif is_reviewer(self.get_object(), self.request):
                prefix = "reviewer_"
            else:
                prefix = "public_"
            return self.serializers[prefix + self.action]
        return self.serializers.get(self.action, self.serializer_class)


class AccessCallReviewTemplateUploadView(ObjectFileUploadView):
    model = AccessCall
    queryset = AccessCall.objects.all()
    file_field = "review_template"
    extensions = ("docx", "doc", "odt", "pdf")
    permission_classes = [
        permissions.IsAuthenticated,
        CoordinatorEditOrDjangoModelPermissions,
    ]


class AccessCallFieldDownloadView(ObjectFileDownloadView):

    model = AccessCall
    permission_classes = [
        permissions.IsAuthenticated,
        CoordinatorEditOrDjangoModelPermissions,
    ]

    def get_file(self):
        if not self.value.template:
            raise Http404
        return self.value.template

    def get_basename(self):
        ext = self.value.template.name.split(".")[-1]
        return f"access_call_{self.value.key}.{ext}"

    def get(self, request, *args, **kwargs):
        self.authenticate(request)
        self.has_permission(request)

        self.object = self.get_object()
        self.value = FormField.objects.get(id=kwargs["value_id"])
        return super(ObjectFileDownloadView, self).get(request, *args, **kwargs)


class AccessCallFieldUploadView(GenericAPIView):

    model = AccessCall
    queryset = AccessCall.objects.all()
    parser_classes = [FileUploadParser]
    permission_classes = [
        permissions.IsAuthenticated,
        CoordinatorEditOrDjangoModelPermissions,
    ]
    extensions = (".odt", ".doc", ".docx", ".pdf")

    def put(self, request, *args, **kwargs):
        value = FormField.objects.get(id=kwargs["value_id"])

        media_root = Path(settings.MEDIA_ROOT)
        instance = self.get_object()
        storage_dir = Path(
            f"{self.model._meta.model_name}/{instance.id}/field_templates"
        )
        os.makedirs(media_root / storage_dir, exist_ok=True)

        filename = request.data["file"].name

        _, ext = os.path.splitext(filename)
        if ext not in self.extensions:
            return Response(
                {"reason": "File extension not supported"},
                status=400,
            )

        file_path = f"{uuid.uuid4()}{ext}"
        written_path = FileSystemStorage(location=media_root / storage_dir).save(
            file_path, request.data["file"]
        )

        value.template.original_file_name = filename
        value.template.name = str(storage_dir / written_path)
        value.save()

        return Response(status=204)


class AccessCallFacilityReviewViewSet(viewsets.ModelViewSet):
    queryset = AccessCallFacilityReview.objects.all().order_by("id")
    serializer_class = AccessCallFacilityReviewSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]



# Generated by Django 5.2.3 on 2025-11-19 15:17

import django.db.models.deletion
import django.utils.timezone
import functools
import ichec_django_core.models.utils
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0003_alter_formfieldvalue_value"),
        ("marinerg_test_access", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="accessapplicationboardreview",
            name="application",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="marinerg_test_access.accessapplication",
            ),
        ),
        migrations.AlterField(
            model_name="accessapplication",
            name="call",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="applications",
                to="marinerg_test_access.accesscall",
            ),
        ),
        migrations.AlterField(
            model_name="accessapplication",
            name="status",
            field=models.CharField(
                choices=[
                    ("CREATED", "Created"),
                    ("SUBMITTED", "Submitted"),
                    ("FACILITY", "Awaiting Facility Review"),
                    ("BOARD", "Awaiting Board Review"),
                    ("DECISION", "Awaiting Decision"),
                    ("ACCEPTED", "Accepted"),
                    ("REJECTED", "Rejected"),
                ],
                default="CREATED",
                max_length=10,
            ),
        ),
        migrations.AlterField(
            model_name="accessapplication",
            name="summary",
            field=models.FileField(
                null=True,
                upload_to=functools.partial(
                    ichec_django_core.models.utils.content_file_name,
                    *("application_summary",),
                    **{},
                ),
            ),
        ),
        migrations.CreateModel(
            name="AccessApplicationBoardDecision",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "decision",
                    models.CharField(
                        choices=[("1", "ACCEPT"), ("2", "REJECT")], max_length=10
                    ),
                ),
                ("comments", models.TextField(blank=True)),
                (
                    "application",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accessapplication",
                    ),
                ),
                (
                    "call",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="marinerg_test_access.accesscall",
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from .access_call import AccessCall, AccessCallFacilityReview
from .application import AccessApplication

from .reviews import (
    FacilityTestReport,
    AccessApplicationFacilityReview,
    AccessApplicationBoardReview,
    AccessApplicationBoardDecision,
)

__all__ = [
    "AccessCall",
    "AccessCallFacilityReview",
    "AccessApplication",
    "AccessApplicationFacilityReview",
    "AccessApplicationBoardReview",
    "AccessApplicationBoardDecision",
    "FacilityTestReport",
]



import functools

from django.db import models
from django.utils import timezone

from ichec_django_core.models import (
    Member,
    TimesStampMixin,
    PopulatedForm,
    content_file_name,
)

from marinerg_facility.models import Facility

from .access_call import AccessCall
from .summary import generate_summary
from .email import send_application_submit_emails


class AccessApplication(TimesStampMixin):

    class Meta:
        verbose_name = "Access Application"
        verbose_name_plural = "Access Application"

    class Status(models.TextChoices):
        CREATED = "CREATED", "Created"
        SUBMITTED = "SUBMITTED", "Submitted"
        AWAITING_FACILITY_REVIEW = "FACILITY", "Awaiting Facility Review"
        AWAITING_BOARD_REVIEW = "BOARD", "Awaiting Board Review"
        AWAITING_DECISION = "DECISION", "Awaiting Decision"
        ACCEPTED = "ACCEPTED", "Accepted"
        REJECTED = "REJECTED", "Rejected"

    facilities = models.ManyToManyField(
        Facility, related_name="application_choices", blank=True
    )

    chosen_facility = models.ForeignKey(
        Facility, on_delete=models.CASCADE, related_name="applications", null=True
    )

    request_start_date = models.DateTimeField(null=True)
    request_end_date = models.DateTimeField(null=True)
    dates_flexible = models.BooleanField(default=False)

    applicant = models.ForeignKey(Member, on_delete=models.CASCADE)
    call = models.ForeignKey(
        AccessCall, on_delete=models.CASCADE, related_name="applications"
    )

    summary = models.FileField(
        null=True,
        upload_to=functools.partial(content_file_name, "application_summary"),
    )
    status = models.CharField(
        max_length=10, choices=Status.choices, default=Status.CREATED
    )

    form = models.OneToOneField(PopulatedForm, on_delete=models.CASCADE)

    submitted = models.DateTimeField(null=True)
    __last_status = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__last_status = str(self.status)

    def save(self, force_insert=False, force_update=False, *args, **kwargs):

        if (
            str(self.status).lower() == "submitted"
            and self.__last_status.lower() == "created"
        ):
            self.on_submitted()
        self.__last_status = str(self.status)

        super().save(force_insert, force_update, *args, **kwargs)

    def on_submitted(self):

        self.submitted = timezone.now()
        generate_summary(self)
        if False:
            send_application_submit_emails(self)

        if self.call.use_facility_review:
            self.status = "FACILITY"
        else:
            self.status = "BOARD"

    def __str__(self):
        return f"{self.applicant.email} application for '{self.call.title}'"



# Generated by Django 5.2.3 on 2025-12-03 14:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_data_access", "0004_datasettag_dataset_tags"),
        ("marinerg_test_access", "0006_remove_facilitytestreport_dataset_uri_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="facilitytestreport",
            name="datasets",
            field=models.ManyToManyField(
                blank=True,
                related_name="test_reports",
                to="marinerg_data_access.dataset",
                verbose_name="Datasets",
            ),
        ),
    ]



# Generated by Django 5.2.3 on 2025-11-19 17:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("marinerg_facility", "0007_equipmenttag_facilitytag_equipment_tags_and_more"),
        ("marinerg_test_access", "0005_facilitytestreport_facility_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="facilitytestreport",
            name="dataset_uri",
        ),
        migrations.AddField(
            model_name="facilitytestreport",
            name="datasets",
            field=models.ManyToManyField(
                blank=True,
                related_name="test_reports",
                to="marinerg_facility.facility",
                verbose_name="Datasets",
            ),
        ),
    ]



# Generated by Django 5.2.3 on 2025-11-19 15:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "marinerg_test_access",
            "0003_rename_creator_accessapplicationboarddecision_reviewer_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="accessapplicationboarddecision",
            name="call",
        ),
    ]



from django.db import models

import functools

from ichec_django_core.models import (
    Member,
    Form,
    TimesStampMixin,
    DefaultPublicMixin,
    SoftDeleteMixin,
    content_file_name,
)

from marinerg_facility.models import Facility


class AccessCall(TimesStampMixin, DefaultPublicMixin, SoftDeleteMixin):

    class Meta:
        verbose_name = "Access Call"
        verbose_name_plural = "Access Calls"

    class StatusChoice(models.TextChoices):
        OPEN = "OPEN", "Open"
        CLOSED = "CLOSED", "Closed"
        DRAFT = "DRAFT", "Draft"

    title = models.CharField(max_length=200)
    description = models.TextField()
    status = models.CharField(
        max_length=10, choices=StatusChoice.choices, default=StatusChoice.DRAFT
    )
    closing_date = models.DateTimeField()
    coordinator = models.ForeignKey(Member, on_delete=models.CASCADE)
    contact_email = models.CharField(max_length=200, null=True)

    board_chair = models.ForeignKey(
        Member, on_delete=models.CASCADE, related_name="chaired_access_boards"
    )
    board_members = models.ManyToManyField(
        Member,
        verbose_name="list of review board members",
        related_name="access_boards",
        blank=True,
    )

    use_facility_review = models.BooleanField(default=False)

    selectable_facilities = models.ManyToManyField(
        Facility,
        verbose_name="Selectable Facilities",
        related_name="selectable_access_calls",
        blank=True,
    )

    applications_summary = models.FileField(
        null=True,
        upload_to=functools.partial(
            content_file_name, "access_call_applications_summary"
        ),
    )

    review_template = models.FileField(null=True)

    form = models.OneToOneField(Form, on_delete=models.CASCADE)

    def __str__(self):
        return self.title

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__last_status = str(self.status)

    def save(self, force_insert=False, force_update=False, *args, **kwargs):

        if (
            str(self.status).lower() == "closed"
            and self.__last_status.lower() != "closed"
        ):
            self.on_closed()

        super().save(force_insert, force_update, *args, **kwargs)

    def on_closed(self):
        for application in self.applications.all():
            if application.status == "SUBMITTED" or application.status == "FACILITY":
                application.status = "BOARD"
                application.save()

    @property
    def resolved_contact_email(self):
        if self.contact_email:
            return self.contact_email
        else:
            return self.coordinator.email

    def get_field_by_key(self, key: str):
        for group in self.form.groups.all():
            for field in group.fields.all():
                if field.key == key:
                    return field
        raise RuntimeError(f"Requested field '{key}' not found")


class AccessCallFacilityReview(TimesStampMixin):

    class Meta:
        verbose_name = "Access Call Facility Review"
        verbose_name_plural = "Access Call Facility Reviews"

    class Decision(models.TextChoices):
        ACCEPT = "ACCEPT", "Accept"
        REJECT = "REJECT", "Reject"

    decision = models.CharField(max_length=10, choices=Decision.choices)
    comments = models.TextField(blank=True)
    call = models.ForeignKey(AccessCall, on_delete=models.CASCADE)
    facility = models.ForeignKey(Facility, on_delete=models.CASCADE)



from rest_framework import serializers
from rest_framework.reverse import reverse

from ichec_django_core.serializers import (
    NestedHyperlinkedModelSerializer,
    FormSerializer,
)

from marinerg_test_access.models import AccessCall, AccessCallFacilityReview


class AccessCallBaseSerializer(NestedHyperlinkedModelSerializer):

    class Meta:
        model = AccessCall
        fields = NestedHyperlinkedModelSerializer.base_fields + (
            "title",
            "description",
            "status",
            "closing_date",
            "selectable_facilities",
            "resolved_contact_email",
        )
        read_only_fields = NestedHyperlinkedModelSerializer.base_fields + (
            "resolved_contact_email",
        )


class AccessCallListSerializer(AccessCallBaseSerializer):
    pass


class AccessCallPublicDetailSerializer(AccessCallBaseSerializer):
    form = FormSerializer()

    class Meta:
        model = AccessCall
        fields = AccessCallBaseSerializer.Meta.fields + ("form",)
        read_only_fields = AccessCallBaseSerializer.Meta.read_only_fields

    def to_representation(self, instance):
        rep = super().to_representation(instance)

        for group in rep["form"]["groups"]:
            for field in group["fields"]:
                if field["template"]:
                    field["template"] = reverse(
                        "access_call_field_download",
                        args=[instance.id, field["id"]],
                        request=self.context["request"],
                    )
                else:
                    field["template"] = None
        return rep


class AccessCallReviewerDetailSerializer(AccessCallPublicDetailSerializer):

    review_template = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="accesscall-review_template"
    )

    class Meta:
        model = AccessCall
        fields = AccessCallPublicDetailSerializer.Meta.fields + (
            "coordinator",
            "board_chair",
            "review_template",
        )
        read_only_fields = AccessCallPublicDetailSerializer.Meta.read_only_fields + (
            "review_template",
        )

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.review_template:
            rep["review_template"] = None
        return rep


class AccessCallDetailSerializer(AccessCallReviewerDetailSerializer):

    applications_summary = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="accesscall-applications_summary"
    )

    class Meta:
        model = AccessCall
        fields = AccessCallReviewerDetailSerializer.Meta.fields + (
            "board_members",
            "public",
            "use_facility_review",
            "applications_summary",
        )
        read_only_fields = AccessCallReviewerDetailSerializer.Meta.read_only_fields + (
            "applications_summary",
        )

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.applications_summary:
            rep["applications_summary"] = None
        return rep

    def create(self, validated_data):

        form = validated_data.pop("form")
        form_instance = FormSerializer().create(form)

        many_to_many = self.pop_many_to_many(validated_data)
        instance = AccessCall.objects.create(form=form_instance, **validated_data)
        self.add_many_to_many(instance, many_to_many)
        return instance

    def update(self, instance, validated_data):
        form = validated_data.pop("form")

        instance = super().update(instance, validated_data)

        serializer = FormSerializer()
        serializer.update(instance.form, form)

        return instance


class AccessCallFacilityReviewSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = AccessCallFacilityReview
        fields = ["decision", "comments", "call", "facility"]



# Generated by Django 5.2.3 on 2026-02-05 19:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "marinerg_test_access",
            "0008_accesscall_active_accesscall_deleted_at_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="accessapplicationboardreview",
            name="supporting_document",
            field=models.FileField(null=True, upload_to=""),
        ),
        migrations.AddField(
            model_name="accesscall",
            name="contact_email",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.AddField(
            model_name="accesscall",
            name="review_template",
            field=models.FileField(null=True, upload_to=""),
        ),
        migrations.AddField(
            model_name="accesscall",
            name="use_facility_review",
            field=models.BooleanField(default=False),
        ),
    ]



from datetime import datetime

from pydantic import BaseModel, Extra
from ichec_django_core.client import (
    Identifiable,
    Timestamped,
    FormCreate,
    FormDetail,
    FormFieldDetail,
    register_types,
    PopulatedFormCreate,
    PopulatedFormDetail,
)


class AccessCallBase(BaseModel, frozen=True):

    title: str
    description: str
    status: str
    closing_date: datetime
    selectable_facilities: list[str] = []


class AccessCallCreate(AccessCallBase, frozen=True):

    form: FormCreate
    use_facility_review: bool
    public: bool
    coordinator: str
    board_chair: str
    board_members: list[str] = []


class AccessCallPublicDetail(Timestamped, AccessCallBase, frozen=True):
    resolved_contact_email: str
    form: FormDetail

    class Config:
        extra = Extra.forbid

    def get_field(self, key: str) -> FormFieldDetail | None:
        for group in self.form.groups:
            for field in group.fields:
                if field.key == key:
                    return field
        return None


class AccessCallReviewerDetail(AccessCallPublicDetail, frozen=True):
    coordinator: str
    board_chair: str
    review_template: str | None


class AccessCallDetail(AccessCallReviewerDetail, frozen=True):

    board_members: list[str] = []
    applications_summary: str | None
    use_facility_review: bool
    public: bool


class AccessCallList(Timestamped, AccessCallBase, frozen=True):

    pass


class ApplicationFieldValueBase(BaseModel, frozen=True):

    value: str
    field: str
    asset: str | None = None


class ApplicationFieldValueCreate(ApplicationFieldValueBase, frozen=True):
    pass


class ApplicationFieldValueDetail(Identifiable, ApplicationFieldValueBase, frozen=True):
    pass


class AccessApplicationBase(BaseModel, frozen=True):
    call: str
    facilities: list[str] = []
    chosen_facility: str | None = None
    request_start_date: datetime | None = None
    request_end_date: datetime | None = None
    dates_flexible: bool = True
    status: str = "SUBMITTED"


class AccessApplicationCreate(AccessApplicationBase, frozen=True):
    form: PopulatedFormCreate


class AccessApplicationDetail(Timestamped, AccessApplicationBase, frozen=True):
    applicant: str
    submitted: datetime | None = None
    form: PopulatedFormDetail


register_types(
    "access_calls",
    {
        "create": AccessCallCreate,
        "detail-public": AccessCallPublicDetail,
        "detail-reviewer": AccessCallReviewerDetail,
        "detail": AccessCallDetail,
        "list": AccessCallList,
    },
)

register_types(
    "access_applications",
    {"create": AccessApplicationCreate, "detail": AccessApplicationDetail},
)



from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string


def send_application_submit_emails(application):

    context = {
        "domain": "http://localhost:4200",
        "application_id": application.id,
        "call": application.call.title,
        "facility_name": (
            application.chosen_facility.name if application.chosen_facility else ""
        ),
    }

    # Email applicant and facility
    emails_dir = "marinerg_test_access/emails"

    template = emails_dir + "/invite_facility_application_review"
    text = render_to_string(template + ".txt", context=context)
    html = render_to_string(template + ".html", context=context)

    msg = EmailMultiAlternatives(
        subject="MARINEG-i: New Access Application for Review",
        body=text,
        from_email="from@example.com",
        to=["to@example.com"],
        headers={"List-Unsubscribe": "<mailto:unsub@example.com>"},
    )

    msg.attach_alternative(html, "text/html")
    msg.send()
